@@ -6,10 +6,14 @@ from typing import (
     Collection,
     Dict,
     Iterable,
+    NoReturn,
     Tuple,
     Union,
 )
 
+from eth_typing import (
+    HexStr,
+)
 from eth_utils.curried import (
     apply_formatter_at_index,
     apply_formatter_if,
@@ -17,6 +21,7 @@ from eth_utils.curried import (
     apply_formatters_to_dict,
     apply_formatters_to_sequence,
     apply_one_of_formatters,
+    is_0x_prefixed,
     is_address,
     is_bytes,
     is_dict,
@@ -68,9 +73,15 @@ from web3._utils.rpc_abi import (
 from web3.datastructures import (
     AttributeDict,
 )
+from web3.exceptions import (
+    BlockNotFound,
+    TransactionNotFound,
+)
 from web3.types import (
+    BlockIdentifier,
     RPCEndpoint,
     TReturn,
+    _Hash32,
 )
 
 
@@ -81,6 +92,7 @@ def bytes_to_ascii(value: bytes) -> str:
 to_ascii_if_bytes = apply_formatter_if(is_bytes, bytes_to_ascii)
 to_integer_if_hex = apply_formatter_if(is_string, hex_to_integer)
 block_number_formatter = apply_formatter_if(is_integer, integer_to_hex)
+to_hex_if_integer = apply_formatter_if(is_integer, integer_to_hex)
 
 
 is_false = partial(operator.is_, False)
@@ -91,7 +103,7 @@ is_not_null = complement(is_null)
 
 @curry
 def to_hexbytes(
-    num_bytes: int, val: Union[str, int, bytes], variable_length: bool=False
+    num_bytes: int, val: Union[str, int, bytes], variable_length: bool = False
 ) -> HexBytes:
     if isinstance(val, (str, int, bytes)):
         result = HexBytes(val)
@@ -348,13 +360,13 @@ PYTHONIC_REQUEST_FORMATTERS: Dict[RPCEndpoint, Callable[..., Any]] = {
     RPC.eth_getStorageAt: apply_formatter_at_index(block_number_formatter, 2),
     RPC.eth_getTransactionByBlockNumberAndIndex: compose(
         apply_formatter_at_index(block_number_formatter, 0),
-        apply_formatter_at_index(integer_to_hex, 1),
+        apply_formatter_at_index(to_hex_if_integer, 1),
     ),
     RPC.eth_getTransactionCount: apply_formatter_at_index(block_number_formatter, 1),
     RPC.eth_getUncleCountByBlockNumber: apply_formatter_at_index(block_number_formatter, 0),
     RPC.eth_getUncleByBlockNumberAndIndex: compose(
         apply_formatter_at_index(block_number_formatter, 0),
-        apply_formatter_at_index(integer_to_hex, 1),
+        apply_formatter_at_index(to_hex_if_integer, 1),
     ),
     RPC.eth_getUncleByBlockHashAndIndex: apply_formatter_at_index(integer_to_hex, 1),
     RPC.eth_newFilter: apply_formatter_at_index(filter_params_formatter, 0),
@@ -394,6 +406,7 @@ PYTHONIC_RESULT_FORMATTERS: Dict[RPCEndpoint, Callable[..., Any]] = {
     RPC.eth_blockNumber: to_integer_if_hex,
     RPC.eth_chainId: to_integer_if_hex,
     RPC.eth_coinbase: to_checksum_address,
+    RPC.eth_call: HexBytes,
     RPC.eth_estimateGas: to_integer_if_hex,
     RPC.eth_gasPrice: to_integer_if_hex,
     RPC.eth_getBalance: to_integer_if_hex,
@@ -425,8 +438,8 @@ PYTHONIC_RESULT_FORMATTERS: Dict[RPCEndpoint, Callable[..., Any]] = {
     RPC.eth_getUncleCountByBlockNumber: to_integer_if_hex,
     RPC.eth_hashrate: to_integer_if_hex,
     RPC.eth_protocolVersion: compose(
+        apply_formatter_if(is_0x_prefixed, to_integer_if_hex),
         apply_formatter_if(is_integer, str),
-        to_integer_if_hex,
     ),
     RPC.eth_sendRawTransaction: to_hexbytes(32),
     RPC.eth_sendTransaction: to_hexbytes(32),
@@ -486,14 +499,63 @@ def get_request_formatters(
     method_name: Union[RPCEndpoint, Callable[..., RPCEndpoint]]
 ) -> Dict[str, Callable[..., Any]]:
     request_formatter_maps = (
+        ABI_REQUEST_FORMATTERS,
+        # METHOD_NORMALIZERS needs to be after ABI_REQUEST_FORMATTERS
+        # so that eth_getLogs's apply_formatter_at_index formatter
+        # is applied to the whole address
+        # rather than on the first byte of the address
         METHOD_NORMALIZERS,
         PYTHONIC_REQUEST_FORMATTERS,
-        ABI_REQUEST_FORMATTERS
     )
     formatters = combine_formatters(request_formatter_maps, method_name)
     return compose(*formatters)
 
 
+def raise_block_not_found(params: Tuple[BlockIdentifier, bool]) -> NoReturn:
+    block_identifier = params[0]
+    raise BlockNotFound(f"Block with id: {block_identifier} not found.")
+
+
+def raise_block_not_found_for_uncle_at_index(
+    params: Tuple[BlockIdentifier, Union[HexStr, int]]
+) -> NoReturn:
+    block_identifier = params[0]
+    uncle_index = to_integer_if_hex(params[1])
+    raise BlockNotFound(
+        f"Uncle at index: {uncle_index} of block with id: {block_identifier} not found."
+    )
+
+
+def raise_transaction_not_found(params: Tuple[_Hash32]) -> NoReturn:
+    transaction_hash = params[0]
+    raise TransactionNotFound(f"Transaction with hash: {transaction_hash} not found.")
+
+
+def raise_transaction_not_found_with_index(params: Tuple[BlockIdentifier, int]) -> NoReturn:
+    block_identifier = params[0]
+    transaction_index = to_integer_if_hex(params[1])
+    raise TransactionNotFound(
+        f"Transaction index: {transaction_index} "
+        f"on block id: {block_identifier} not found."
+    )
+
+
+NULL_RESULT_FORMATTERS: Dict[RPCEndpoint, Callable[..., Any]] = {
+    RPC.eth_getBlockByHash: raise_block_not_found,
+    RPC.eth_getBlockByNumber: raise_block_not_found,
+    RPC.eth_getBlockTransactionCountByHash: raise_block_not_found,
+    RPC.eth_getBlockTransactionCountByNumber: raise_block_not_found,
+    RPC.eth_getUncleCountByBlockHash: raise_block_not_found,
+    RPC.eth_getUncleCountByBlockNumber: raise_block_not_found,
+    RPC.eth_getUncleByBlockHashAndIndex: raise_block_not_found_for_uncle_at_index,
+    RPC.eth_getUncleByBlockNumberAndIndex: raise_block_not_found_for_uncle_at_index,
+    RPC.eth_getTransactionByHash: raise_transaction_not_found,
+    RPC.eth_getTransactionByBlockHashAndIndex: raise_transaction_not_found_with_index,
+    RPC.eth_getTransactionByBlockNumberAndIndex: raise_transaction_not_found_with_index,
+    RPC.eth_getTransactionReceipt: raise_transaction_not_found,
+}
+
+
 def get_result_formatters(
     method_name: Union[RPCEndpoint, Callable[..., RPCEndpoint]]
 ) -> Dict[str, Callable[..., Any]]:
@@ -503,15 +565,14 @@ def get_result_formatters(
     )
     attrdict_formatter = apply_formatter_if(is_dict and not_attrdict, AttributeDict.recursive)
 
-    return compose(*formatters, attrdict_formatter)
+    return compose(attrdict_formatter, *formatters)
 
 
 def get_error_formatters(
     method_name: Union[RPCEndpoint, Callable[..., RPCEndpoint]]
 ) -> Dict[str, Callable[..., Any]]:
     #  Note error formatters work on the full response dict
-    # TODO - test this function
-    error_formatter_maps = ()
+    error_formatter_maps = (NULL_RESULT_FORMATTERS,)
     formatters = combine_formatters(error_formatter_maps, method_name)
 
     return compose(*formatters)
